==> go-ld-cbor-v0.0.1.tar.bz2: go-ld-cbor-v0.0.1/store.go <==
package cbornode

import (
	"bytes"
	"context"
	"fmt"

	mh "github.com/multiformats/go-multihash"
	recbor "github.com/polydawn/refmt/cbor"
	atlas "github.com/polydawn/refmt/obj/atlas"
	cbg "github.com/whyrusleeping/cbor-gen"
	block "gitlab.dms3.io/dms3/public/go-block-format"
	cid "gitlab.dms3.io/dms3/public/go-cid"
)

// Dms3LdStore wraps a Blockstore and provides an interface for storing and retrieving CBOR encoded data.
type Dms3LdStore interface {
	Get(ctx context.Context, c cid.Cid, out interface{}) error
	Put(ctx context.Context, v interface{}) (cid.Cid, error)
}

// Dms3LdBlockstore defines a subset of the go-dms3-blockstore Blockstore interface providing methods
// for storing and retrieving block-centered data.
type Dms3LdBlockstore interface {
	Get(cid.Cid) (block.Block, error)
	Put(block.Block) error
}

// Dms3LdBlockstoreViewer is a trait that enables zero-copy access to blocks in
// a blockstore.
type Dms3LdBlockstoreViewer interface {
	// View provides zero-copy access to blocks in a blockstore. The callback
	// function will be invoked with the value for the key. The user MUST not
	// modify the byte array, as it could be memory-mapped.
	View(cid.Cid, func([]byte) error) error
}

// BasicDms3LdStore wraps and Dms3LdBlockstore and implements the Dms3LdStore interface.
type BasicDms3LdStore struct {
	Blocks Dms3LdBlockstore
	Viewer Dms3LdBlockstoreViewer

	Atlas *atlas.Atlas
}

var _ Dms3LdStore = &BasicDms3LdStore{}

// NewCborStore returns an Dms3LdStore implementation backed by the provided Dms3LdBlockstore.
func NewCborStore(bs Dms3LdBlockstore) *BasicDms3LdStore {
	viewer, _ := bs.(Dms3LdBlockstoreViewer)
	return &BasicDms3LdStore{Blocks: bs, Viewer: viewer}
}

// Get reads and unmarshals the content at `c` into `out`.
func (s *BasicDms3LdStore) Get(ctx context.Context, c cid.Cid, out interface{}) error {
	if s.Viewer != nil {
		// zero-copy path.
		return s.Viewer.View(c, func(b []byte) error {
			return s.decode(b, out)
		})
	}

	blk, err := s.Blocks.Get(c)
	if err != nil {
		return err
	}
	return s.decode(blk.RawData(), out)
}

func (s *BasicDms3LdStore) decode(b []byte, out interface{}) error {
	cu, ok := out.(cbg.CBORUnmarshaler)
	if ok {
		if err := cu.UnmarshalCBOR(bytes.NewReader(b)); err != nil {
			return NewSerializationError(err)
		}
		return nil
	}

	if s.Atlas == nil {
		return DecodeInto(b, out)
	} else {
		return recbor.UnmarshalAtlased(recbor.DecodeOptions{}, b, out, *s.Atlas)
	}
}

type cidProvider interface {
	Cid() cid.Cid
}

// Put marshals and writes content `v` to the backing blockstore returning its CID.
func (s *BasicDms3LdStore) Put(ctx context.Context, v interface{}) (cid.Cid, error) {
	mhType := uint64(mh.BLAKE2B_MIN + 31)
	mhLen := -1
	codec := uint64(cid.DagCBOR)

	var expCid cid.Cid
	if c, ok := v.(cidProvider); ok {
		expCid := c.Cid()
		pref := expCid.Prefix()
		mhType = pref.MhType
		mhLen = pref.MhLength
		codec = pref.Codec
	}

	cm, ok := v.(cbg.CBORMarshaler)
	if ok {
		buf := new(bytes.Buffer)
		if err := cm.MarshalCBOR(buf); err != nil {
			return cid.Undef, NewSerializationError(err)
		}

		pref := cid.Prefix{
			Codec:    codec,
			MhType:   mhType,
			MhLength: mhLen,
			Version:  1,
		}
		c, err := pref.Sum(buf.Bytes())
		if err != nil {
			return cid.Undef, err
		}

		blk, err := block.NewBlockWithCid(buf.Bytes(), c)
		if err != nil {
			return cid.Undef, err
		}

		if err := s.Blocks.Put(blk); err != nil {
			return cid.Undef, err
		}

		blkCid := blk.Cid()
		if expCid != cid.Undef && blkCid != expCid {
			return cid.Undef, fmt.Errorf("your object is not being serialized the way it expects to")
		}

		return blkCid, nil
	}

	nd, err := WrapObject(v, mhType, mhLen)
	if err != nil {
		return cid.Undef, err
	}

	if err := s.Blocks.Put(nd); err != nil {
		return cid.Undef, err
	}

	ndCid := nd.Cid()
	if expCid != cid.Undef && ndCid != expCid {
		return cid.Undef, fmt.Errorf("your object is not being serialized the way it expects to")
	}

	return ndCid, nil
}

func NewSerializationError(err error) error {
	return SerializationError{err}
}

type SerializationError struct {
	err error
}

func (se SerializationError) Error() string {
	return se.err.Error()
}

func (se SerializationError) Unwrap() error {
	return se.err
}

func (se SerializationError) Is(o error) bool {
	_, ok := o.(*SerializationError)
	return ok
}

func NewMemCborStore() Dms3LdStore {
	return NewCborStore(newMockBlocks())
}

type mockBlocks struct {
	data map[cid.Cid]block.Block
}

func newMockBlocks() *mockBlocks {
	return &mockBlocks{make(map[cid.Cid]block.Block)}
}

func (mb *mockBlocks) Get(c cid.Cid) (block.Block, error) {
	d, ok := mb.data[c]
	if ok {
		return d, nil
	}
	return nil, fmt.Errorf("Not Found")
}

func (mb *mockBlocks) Put(b block.Block) error {
	mb.data[b.Cid()] = b
	return nil
}

==> go-ld-cbor-v0.0.3.tar.bz2: go-ld-cbor-v0.0.3/store.go <==
package cbornode

import (
	"bytes"
	"context"
	"fmt"

	mh "github.com/multiformats/go-multihash"
	recbor "github.com/polydawn/refmt/cbor"
	atlas "github.com/polydawn/refmt/obj/atlas"
	cbg "gitlab.dms3.io/dms3/cbor-gen"
	block "gitlab.dms3.io/dms3/go-block-format"
	cid "gitlab.dms3.io/dms3/go-cid"
)

// LdStore wraps a Blockstore and provides an interface for storing and retrieving CBOR encoded data.
type LdStore interface {
	Get(ctx context.Context, c cid.Cid, out interface{}) error
	Put(ctx context.Context, v interface{}) (cid.Cid, error)
}

// LdBlockstore defines a subset of the go-dms3-blockstore Blockstore interface providing methods
// for storing and retrieving block-centered data.
type LdBlockstore interface {
	Get(cid.Cid) (block.Block, error)
	Put(block.Block) error
}

// LdBlockstoreViewer is a trait that enables zero-copy access to blocks in
// a blockstore.
type LdBlockstoreViewer interface {
	// View provides zero-copy access to blocks in a blockstore. The callback
	// function will be invoked with the value for the key. The user MUST not
	// modify the byte array, as it could be memory-mapped.
	View(cid.Cid, func([]byte) error) error
}

// BasicLdStore wraps and LdBlockstore and implements the LdStore interface.
type BasicLdStore struct {
	Blocks LdBlockstore
	Viewer LdBlockstoreViewer

	Atlas *atlas.Atlas
}

var _ LdStore = &BasicLdStore{}

// NewCborStore returns an LdStore implementation backed by the provided LdBlockstore.
func NewCborStore(bs LdBlockstore) *BasicLdStore {
	viewer, _ := bs.(LdBlockstoreViewer)
	return &BasicLdStore{Blocks: bs, Viewer: viewer}
}

// Get reads and unmarshals the content at `c` into `out`.
func (s *BasicLdStore) Get(ctx context.Context, c cid.Cid, out interface{}) error {
	if s.Viewer != nil {
		// zero-copy path.
		return s.Viewer.View(c, func(b []byte) error {
			return s.decode(b, out)
		})
	}

	blk, err := s.Blocks.Get(c)
	if err != nil {
		return err
	}
	return s.decode(blk.RawData(), out)
}

func (s *BasicLdStore) decode(b []byte, out interface{}) error {
	cu, ok := out.(cbg.CBORUnmarshaler)
	if ok {
		if err := cu.UnmarshalCBOR(bytes.NewReader(b)); err != nil {
			return NewSerializationError(err)
		}
		return nil
	}

	if s.Atlas == nil {
		return DecodeInto(b, out)
	} else {
		return recbor.UnmarshalAtlased(recbor.DecodeOptions{}, b, out, *s.Atlas)
	}
}

type cidProvider interface {
	Cid() cid.Cid
}

// Put marshals and writes content `v` to the backing blockstore returning its CID.
func (s *BasicLdStore) Put(ctx context.Context, v interface{}) (cid.Cid, error) {
	mhType := uint64(mh.BLAKE2B_MIN + 31)
	mhLen := -1
	codec := uint64(cid.DagCBOR)

	var expCid cid.Cid
	if c, ok := v.(cidProvider); ok {
		expCid := c.Cid()
		pref := expCid.Prefix()
		mhType = pref.MhType
		mhLen = pref.MhLength
		codec = pref.Codec
	}

	cm, ok := v.(cbg.CBORMarshaler)
	if ok {
		buf := new(bytes.Buffer)
		if err := cm.MarshalCBOR(buf); err != nil {
			return cid.Undef, NewSerializationError(err)
		}

		pref := cid.Prefix{
			Codec:    codec,
			MhType:   mhType,
			MhLength: mhLen,
			Version:  1,
		}
		c, err := pref.Sum(buf.Bytes())
		if err != nil {
			return cid.Undef, err
		}

		blk, err := block.NewBlockWithCid(buf.Bytes(), c)
		if err != nil {
			return cid.Undef, err
		}

		if err := s.Blocks.Put(blk); err != nil {
			return cid.Undef, err
		}

		blkCid := blk.Cid()
		if expCid != cid.Undef && blkCid != expCid {
			return cid.Undef, fmt.Errorf("your object is not being serialized the way it expects to")
		}

		return blkCid, nil
	}

	nd, err := WrapObject(v, mhType, mhLen)
	if err != nil {
		return cid.Undef, err
	}

	if err := s.Blocks.Put(nd); err != nil {
		return cid.Undef, err
	}

	ndCid := nd.Cid()
	if expCid != cid.Undef && ndCid != expCid {
		return cid.Undef, fmt.Errorf("your object is not being serialized the way it expects to")
	}

	return ndCid, nil
}

func NewSerializationError(err error) error {
	return SerializationError{err}
}

type SerializationError struct {
	err error
}

func (se SerializationError) Error() string {
	return se.err.Error()
}

func (se SerializationError) Unwrap() error {
	return se.err
}

func (se SerializationError) Is(o error) bool {
	_, ok := o.(*SerializationError)
	return ok
}

func NewMemCborStore() LdStore {
	return NewCborStore(newMockBlocks())
}

type mockBlocks struct {
	data map[cid.Cid]block.Block
}

func newMockBlocks() *mockBlocks {
	return &mockBlocks{make(map[cid.Cid]block.Block)}
}

func (mb *mockBlocks) Get(c cid.Cid) (block.Block, error) {
	d, ok := mb.data[c]
	if ok {
		return d, nil
	}
	return nil, fmt.Errorf("Not Found")
}

func (mb *mockBlocks) Put(b block.Block) error {
	mb.data[b.Cid()] = b
	return nil
}
